/*
Using single one hotkey of Attach/Detach. Works with:
Edit Poly
Editable poly
Edit mesh
Editable mesh
Edit Spline modifer
SplineShape
line
UnwrapUVW

 Script: UniAttach-Detacher
 Author: Nikolay Litvinov
 Created in the 3dsMax 2012
*/

macroScript UniAttach Category:"Niklit Scripts" Tooltip:"UniAttach"
(
------------------------ FUNCTIONS ------------------------
fn FN_MultiAttach = (
achLst=for o in selection collect o
Anum=achLst.count
for o=1 to (Anum-1) do (converttopoly achLst[Anum]; polyop.attach achLst[Anum] achLst[o])
pushprompt (achLst.count as string + " Objects attached")
)

fn FN_Filter_GSobj = (
	try(
	arO = For o in selection collect superclassof o
	if (findItem arO shape)!=0 and (findItem arO GeometryClass)!=0
		then (messageBox ("Please select only Geometry or only Shapes")
				clearSelection())

	if (findItem arO shape)==0 then FN_MultiAttach()
	else (select selection[1]
		splineOps.attachMultiple selection[1])
	)catch()
)
------------------------ SCRIPT ------------------------

 if getCommandPanelTaskMode() != #modify then (max modify mode)
local gco = modPanel.getCurrentObject()
 Case classOf gco of
				(
	   Edit_Poly:	macros.run "Editable Polygon Object" "EPoly_Attach"
   Editable_poly:	macros.run "Editable Polygon Object" "EPoly_Attach"
	   Edit_mesh:	macros.run "Editable Mesh Object" "EMesh_Attach"
   Editable_mesh:	macros.run "Editable Mesh Object" "EMesh_Attach"
   	 		line:	macros.run "Editable Spline Object" "ESpline_Attach"
	 SplineShape:	macros.run "Editable Spline Object" "ESpline_Attach"
	 Edit_Spline:	macros.run "Editable Spline Object" "ESpline_Attach"
	  unwrap_uvw:	gco.stitchVertsDialog()
				)
	if selection.count > 1 do FN_Filter_GSobj()
)
-------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------

macroScript UniDetach Category:"Niklit Scripts" Tooltip:"UniDetach"
(
------------------------ FUNCTIONS ------------------------
Fn FN_ResetWirecolor = (try($.material=undefined
	$.wirecolor = color (random 0 255) (random 0 255) (random 0 255)) Catch()	)

fn selectLastCreatedObj =(objID = amax (for o in objects collect o.inode.handle)
select (maxOps.getNodeByHandle objID))

fn Explode_El_to_Objects=(
'Objects 0'= objects as array
 while(getFaceSelection $.mesh).numberset != 0
	do(
		El = polyOp.getElementsUsingFace $ #{1}
		polyOp.detachFaces $ El asNode:true name: (uniqueName ("Exploded "+$.name as string))
	  )
	delete $
	'Exploded objects' = for o in objects where findItem 'Objects 0' o == 0 collect o
	for o in 'Exploded objects' do 
	(centerPivot o; worldAlignPivot o; o.wirecolor = color (random 0 255) (random 0 255) (random 0 255))
	 select 'Exploded objects'[1]
)

fn FN_so4_operationEP =('Objects 0'= objects as array
	numV0=$.GetNumVertices()
	if getNumFaces $.mesh==(getFaceSelection $.mesh).numberset then (Explode_El_to_Objects())
	else (macros.run "Editable Polygon Object" "EPoly_Detach"
	 	if 'Objects 0'.count != (objects as array).count or $.GetNumVertices()!=numV0 do
	 		(
	 		if Sel_LastObj==true do (selectLastCreatedObj(); CenterPivot $)
	 		if ResetMtl_UniDetach==true do FN_ResetWirecolor()
	 		)
	 	)
	)
fn FN_so4_operationEtP fco =('Objects 0'= objects as array
	numV0=fco.GetNumVertices()
	fco.PopupDialog #DetachFace
 	if 'Objects 0'.count != (objects as array).count or fco.GetNumVertices()!=numV0 do
 		(
 		if Sel_LastObj==true do (selectLastCreatedObj(); CenterPivot $)
 		if ResetMtl_UniDetach==true do FN_ResetWirecolor()
 		)
 	)


fn createShapeFromEdges_fn=(try (polyop.createShape $ (getEdgeSelection $))
catch (Btn_hwnd=for i in windows.getChildrenHWND #max where i[5]=="Create Shape" do exit with i[1]; UIAccessor.PressButton Btn_hwnd)
select (maxOps.getNodeByHandle (amax (for o in objects collect o.inode.handle)))
$.pivot=$.center)
------------------------ SCRIPT ------------------------
if subobjectlevel==undefined or subobjectlevel==0 
then (sel0=selection as array
maxOps.cloneNodes $ cloneType:#copy newNodes:&nnl
hide sel0
select nnl)
else
(
 if getCommandPanelTaskMode() != #modify then max modify mode
 local gco = modPanel.getCurrentObject()
 	Case classOf gco of
	(
Editable_poly:	Case subobjectLevel of
				(
				 1: macros.run "Editable Polygon Object" "EPoly_VBreak"
				 2: $.EditablePoly.splitEdges ()
				 3: createShapeFromEdges_fn()
				 4: FN_so4_operationEP()
				 5: FN_so4_operationEP()
				)
	Edit_Poly:	Case subobjectLevel of
				(
				 1: gco.SetOperation #BreakVertex
				 2: gco.ButtonOp #SplitEdges
				 3: createShapeFromEdges_fn()
				 4: FN_so4_operationEtP gco
				 5: FN_so4_operationEtP gco
				)
	Edit_mesh:	if subObjectLevel==1 
	 			then macros.run "Editable Mesh Object" "EMesh_VBreak"
				else macros.run "Editable Mesh Object" "EMesh_Detach"
Editable_mesh:	if subObjectLevel==1
	 			then macros.run "Editable Mesh Object" "EMesh_VBreak"
				else macros.run "Editable Mesh Object" "EMesh_Detach"
	   line: Case subobjectLevel of
				(
				 1:	macros.run "Editable Spline Object" "ESpline_Break"
				 2: (macros.run "Editable Spline Object" "ESpline_Detach_Segment"
				 UIAccessor.PressDefaultButton())
				 3: (macros.run "Editable Spline Object" "ESpline_Detach_spline"
				 UIAccessor.PressDefaultButton())
				)
SplineShape: Case subobjectLevel of
				(
				 1: macros.run "Editable Spline Object" "ESpline_Break"
				 2: (macros.run "Editable Spline Object" "ESpline_Detach_Segment"
				 UIAccessor.PressDefaultButton())
				 3: (macros.run "Editable Spline Object" "ESpline_Detach_spline"
				 UIAccessor.PressDefaultButton())
				)
Edit_Spline: Case subobjectLevel of
			(
			 1: splineOps.startBreak gco	
			 2: (splineOps.detach gco
			 	UIAccessor.PressDefaultButton())
			 3: (splineOps.detach gco
			 	UIAccessor.PressDefaultButton())
			)
 Unwrap_UVW: gco.breakSelected()
	)
)
)

macroScript UniDetach_ResetMtl_macro Category:"Niklit Scripts" Tooltip:"UniDetach ResetMtl"
icon:#("Material_Modifiers",3)
(
	global ResetMtl_UniDetach
	on ischecked return ResetMtl_UniDetach
	on Execute do ResetMtl_UniDetach = if ResetMtl_UniDetach != true then true else false
)

macroScript UniDetach_Sel_LastObj Category:"Niklit Scripts" Tooltip:"Select LastObj Switcher"
icon:#("enss_tools",9)
(
	global Sel_LastObj
	on ischecked return Sel_LastObj
	on Execute do Sel_LastObj = if Sel_LastObj != true then true else false
)